% Standard Delay Effekt a Zenei Jelfeldolgozas targyhoz
% C. Bank Balazs, 2009-2020.
%Standard effect from Zolzer DAFx book
%vector shifting implementation - easy to understand, but can be very slow for long delays
%(the delayeffekt2 is much faster)

Fs=44100;

L=2*Fs;
[in,Fs]=audioread('concert_guitar.wav');
%[in,Fs]=audioread('drum_bass.wav');
%[in,Fs]=audioread('soprano_sax.wav');
in=in(1:L);


%in=sin(2*pi*260/Fs*[1:L]);
out=zeros(1,L);


%parameters:
%BL: bleach - original signal
%FF: feedforward path with delay
%FB: feedback
%DMIN: minimum delay in ms
%DEPTH: depth of delay modulation in ms
%MODTYPE: modulation type SINE or NOISE (lowpass filtered noise)
%MODF: sinusoidal mod. freq. in Hz if SINE, or cutoff frequency if NOISE

%define constants
SINE=0;
NOISE=1;


%CHOOSE THE EFFECT: UNCOMMENT ONE OF THE FOLLOWING LINES

%NO EFFECT
BL=1; FF=0; FB=0; DMIN=50; DEPTH=0; MODTYPE=SINE; MODF=0;

%DELAY TYPE
% BL=1; FF=1; FB=0; DMIN=50; DEPTH=0; MODTYPE=SINE; MODF=0;  %FIR COMB FILTER - SLAPBACK
% BL=1; FF=0; FB=0.9; DMIN=50; DEPTH=0; MODTYPE=SINE; MODF=0;  %IIR COMB FILTER - ECHO
% BL=1; FF=-0.9; FB=0.9; DMIN=50; DEPTH=0; MODTYPE=SINE; MODF=0;  %ALLPASS COMB FILTER

%MODULATION TYPE
%BL=0;   FF=1;   FB=0;    DMIN=0;   DEPTH=3;   MODTYPE=SINE;  MODF=5;  %VIBRATO
%BL=0.7; FF=0.7; FB=0;    DMIN=0;   DEPTH=2;   MODTYPE=SINE;  MODF=1;  %FLANGER
%BL=0.7; FF=0.7; FB=0.7;  DMIN=0;   DEPTH=2;   MODTYPE=SINE;  MODF=1;  %FEEDBACKED FLANGER
%BL=0.7; FF=1.0; FB=0;    DMIN=10;  DEPTH=10;  MODTYPE=NOISE; MODF=5;  %CHORUS
%BL=0.7; FF=1.0; FB=-0.7; DMIN=10;  DEPTH=10;  MODTYPE=NOISE; MODF=5;  %WHITE CHORUS
%BL=0.7; FF=0.7; FB=0;    DMIN=100; DEPTH=100; MODTYPE=NOISE; MODF=5;  %DOUBLING


%generate the delay length in advance for each sample instant - in a
%realtime effect, you would generate them in the main loop, but this is
%faster in matlab

if MODTYPE==SINE,
    Dt=DMIN+DEPTH/2+DEPTH/2*sin(2*pi*(MODF/Fs)*[1:L]); %the required delay in ms
end;

if MODTYPE==NOISE,
    whitenoise=randn(1,L); %generate gaussian distribution white noise
    [B,A]=butter(2,MODF/(Fs/2));% designing a 2nd order lowpass with cutoff Freq. MODF
    lfnoise=filter(B,A,whitenoise);
    Dt=DMIN+DEPTH/2+DEPTH/2*lfnoise; %the required delay in ms
end;
  
D=Fs*Dt/1000+1; %the required delay in samples  - we need to add 1 because the vectors in matlab start with index 1 
Dint=floor(D); % the integer part of the delay
Dfract=D-Dint; % the fractional part of the delay


%initializing variables
DL=round(Fs*(DMIN+DEPTH)/1000+2);
DelayLine=zeros(1,DL);
dout=0;
din=0;


    

for k=1:L,
    
    din=in(k)+FB*dout; %the input of the delay line
    
    DelayLine=[din DelayLine(1:DL-1)]; %shifting the vector to produce delay
    
    
    %calculating the output of the delay line - uncomment one of the lines
    %dout=DelayLine(Dint(k)); %with no interpolation
    dout=(1-Dfract(k))*DelayLine(Dint(k))+Dfract(k)*DelayLine(Dint(k)+1); % with linear interpolation

    out(k)=FF*dout+BL*din;
end;

soundsc(out,Fs);