% Reverb a Zenei Jelfeldolgozas targyhoz
% C. Bank Balazs, 2009-2020.


%feedbacked delay network
%with Householder feedback matrix


Fs=44100;

N=Fs; %length of the computed impulse response - 1 sec.
Input=[1 zeros(1,N-1)];



%one pole loss filters (for details, see Balazs Bank: Physics-based synthesis of the piano)
%decay time: 1/(c1+c3*w^2)
c1=2.5; %hall: 2.5 - 2 sec, small room: 7.5
c3=c1; %this means half decay time at theta=1, i.e., 7kHz


%Choose reverb type - the first line should stay uncommented, then uncomment one of the line 2, 3, 4 if you want room, plate or spring
DLength=[1010 1170 1240 1363 1577 1965 2020 2289]; % big hall ca. 20*50*10 m - mean free path 12 m
%DLength=[round(DLength/6) round(1.0352*DLength/6)]; %small room 3*4*2.6 m - mean free path 2m
%DLength=round(DLength/6); %plate reverb
%DLength=round(DLength(1:2:8)/4); %very bad spring reverb

MaxDL=max(DLength);
DNum=length(DLength); %number of delay length
DMatrix=zeros(DNum,MaxDL); %the delay matrix - we reserve MaxDL memory for all the lines

%parameters for the loop filters

F0=Fs./DLength; % parameters of one pole loss filters (for details, see Balazs Bank: Physics-based synthesis of the piano)
G=1-c1./F0;
A1=-1-F0/(4*c3)+sqrt(F0.*(2*c3+F0/4))/(2*c3);

%uncomment for no decay
%G=1; 
%A1=0;

%parameters for the shaping filter
%		according to JM Jot, AES '92 preprint: 3030

Alpha=c1/(c1+c3*pi^2);
Bshape=[1 -(1-Alpha)/(1+Alpha)]/(1-(1-Alpha)/(1+Alpha));


%constructung a Householder type of feedback matrix:
A=eye(DNum,DNum)-2/DNum*ones(DNum,DNum);

b=ones(DNum,1);
c=(-1).^[1:DNum];

d=1;

OldOutVect=zeros(DNum,1);

clear Output;

%implementation with shifting delays - a circular buffer would be much faster even in Matlab
for l=1:N,
   
   OutVect=DMatrix(:,1).*G'.*(1.+A1')-OldOutVect.*A1'; %first order lowpass filtering
   OldOutVect=OutVect;
	DMatrix=[DMatrix(:,2:MaxDL) zeros(DNum,1)]; %shifting the delay lines to the left
   InVect=A*OutVect+b*Input(l);	%new input vector
   for m=1:DNum,	%writing the input vector to the delay lines 
      DMatrix(m,DLength(m))=InVect(m);
   end;   
   Output(l)=c*OutVect+d*Input(l);	
   
end;

WhiteOutput=fftfilt(Bshape,Output);
plot(Output);
